import { DateCellCalendarIconElement } from './cellsWithTextDiv/dateCell/dateCellCalendarIconElement';
import { ColumnSettingsBorderUtils } from '../../utils/columnSettings/columnSettingsBorderUtils';
import { ColumnSettingsStyleUtils } from '../../utils/columnSettings/columnSettingsStyleUtils';
import { ColumnSettingsWidthUtils } from '../../utils/columnSettings/columnSettingsWidthUtils';
import { DateCellInputElement } from './cellsWithTextDiv/dateCell/dateCellInputElement';
import { GenericElementUtils } from '../../utils/elements/genericElementUtils';
import { CellTextElement } from './cellsWithTextDiv/text/cellTextElement';
import { CheckboxCellElement } from './checkboxCell/checkboxCellElement';
import { CaretDisplayFix } from '../../utils/browser/caretDisplayFix';
export class CellElement {
    // prettier-ignore
    static setDefaultCellStyle(cellElement, cellStyle, customStyle) {
        Object.assign(cellElement.style, cellStyle, customStyle);
    }
    static createBaseCell(isHeader) {
        const cellElement = document.createElement(isHeader ? CellElement.HEADER_TAG : CellElement.DATA_TAG);
        if (isHeader)
            cellElement.classList.add(CellElement.HEADER_CELL_CLASS);
        cellElement.classList.add(CellElement.CELL_CLASS);
        return cellElement;
    }
    // prettier-ignore
    static createContentCell(isHeader, cellStyle, customStyle, isUsedAsAButton) {
        const cellElement = CellElement.createBaseCell(isHeader);
        if (isHeader && isUsedAsAButton)
            cellElement.classList.add(GenericElementUtils.NOT_SELECTABLE_CLASS);
        // role for assistive technologies
        cellElement.setAttribute('role', 'textbox');
        // should probably remove border width from headerStyle if cellStyle contains it as it will affect the sizer position
        // but the table looks off when the header and data cells have different border styles so it is not expected that
        // users will desire headerstyle it, hence not implementending headerStyle preprocessing functionality
        CellElement.setDefaultCellStyle(cellElement, cellStyle, customStyle);
        return cellElement;
    }
    static setCursor(textContainerElement, isCellTextEditable) {
        textContainerElement.style.cursor = isCellTextEditable ? 'text' : 'default';
    }
    static prepContentEditable(textContainerElement, isEditable, isUsedAsAButton = false) {
        if (CaretDisplayFix.isIssueBrowser()) {
            if (isEditable)
                CaretDisplayFix.setTabIndex(textContainerElement);
            CaretDisplayFix.removeContentEditable(textContainerElement);
        }
        else {
            textContainerElement.contentEditable = String(isEditable); // REF-15
        }
        if (!isUsedAsAButton)
            CellElement.setCursor(textContainerElement, isEditable);
    }
    // prettier-ignore
    // this is used for cases where element could be the cell element or the text inside a select/label cell
    static getCellElement(element) {
        // if cselect/label cell text or date cell text/input container
        if (element.classList.contains(CellTextElement.CELL_TEXT_DIV_CLASS) ||
            element.classList.contains(DateCellInputElement.DATE_INPUT_CONTAINER_CLASS)) {
            return element.parentElement;
            // if date cell input
        }
        else if (element.classList.contains(DateCellInputElement.DATE_INPUT_CLASS) ||
            element.classList.contains(DateCellCalendarIconElement.CALENDAR_ICON_CONTAINER_CLASS)) {
            return element.parentElement.parentElement;
        }
        // if cell
        return element;
    }
    static getTextElement(element) {
        var _a, _b;
        // if select/label cell or date cell
        if ((_a = element.children[0]) === null || _a === void 0 ? void 0 : _a.classList.contains(CellTextElement.CELL_TEXT_DIV_CLASS)) {
            return element.children[0];
            // if header with icon
        }
        else if ((_b = element.children[1]) === null || _b === void 0 ? void 0 : _b.classList.contains(CellTextElement.CELL_TEXT_DIV_CLASS)) {
            return element.children[1];
            // if checkbox cell
        }
        else if (CheckboxCellElement.isCheckboxCell(element)) {
            return element.children[0];
        }
        return element;
    }
    // The reason why .trim() is used is because innerText/textContent property does not just return the cell text, but
    // additionally the new line characters (\n) which represent <br> elements within the cell that make it difficult
    // to compare cell text to other strings or use them for other programmatic purposes.
    // CAUTION-1 - The returned string should not be used to set text on other cells as .trim() removes \n chars for
    // <br> tags which are used to set the pointer position.
    static getText(element) {
        const checkboxValue = CheckboxCellElement.getValue(element);
        if (checkboxValue !== undefined)
            return checkboxValue;
        return element.innerText.trim();
    }
    // this is used for case where element could be cell element that contains a text div element,
    // hence we need to set the text into the correct container
    // CAUTION-1 - be careful that the text does not come from above method
    static setText(element, text) {
        if (!CheckboxCellElement.setValue(element, text)) {
            const textElement = CellElement.getTextElement(element);
            textElement.innerText = text;
        }
    }
    // set text is optional as some elements may only need to toggle the BR padding
    // prettier-ignore
    static setNewText(at, textContainerElement, text, isCellBeingBuilt, isUndo, setText = true) {
        if (setText)
            CellElement.setText(textContainerElement, text);
        // whilst it is primarily used for firefox - we use it consistently for all browsers
        if (isCellBeingBuilt) {
            // in a timeout as text elements may not be populated upfront (data)
            setTimeout(() => CaretDisplayFix.toggleCellTextBRPadding(at, textContainerElement, isUndo));
        }
        else {
            CaretDisplayFix.toggleCellTextBRPadding(at, textContainerElement, isUndo);
        }
    }
    // REF-36
    static setColumnWidth(at, cellElement, customWidths, defWidths) {
        if (!at._tableElementRef)
            return;
        const widths = (customWidths === null || customWidths === void 0 ? void 0 : customWidths.widths) || (defWidths === null || defWidths === void 0 ? void 0 : defWidths.widths);
        if (widths === null || widths === void 0 ? void 0 : widths.staticWidth) {
            ColumnSettingsWidthUtils.updateColumnWidth(at, cellElement, widths, true);
        }
        else if (widths === null || widths === void 0 ? void 0 : widths.initialWidth) {
            const result = ColumnSettingsWidthUtils.getSettingsWidthNumber(at._tableElementRef, widths, false);
            cellElement.style.width = `${result.number}px`;
        }
        else {
            cellElement.style.width = `${at._tableDimensions.newColumnWidth}px`;
        }
    }
    // prettier-ignore
    static createCellElement(at, text, columnIndex, isHeader) {
        var _a, _b, _c;
        const { _defaultColumnsSettings: { cellStyle, headerStyles }, _columnsDetails } = at;
        const columnDetails = _columnsDetails[columnIndex];
        const isOpenViaCellClick = (_c = (_b = (_a = at._defaultColumnsSettings.columnDropdown) === null || _a === void 0 ? void 0 : _a.displaySettings) === null || _b === void 0 ? void 0 : _b.openMethod) === null || _c === void 0 ? void 0 : _c.cellClick;
        const cellElement = CellElement.createContentCell(isHeader, cellStyle, isHeader ? headerStyles === null || headerStyles === void 0 ? void 0 : headerStyles.default : {}, isOpenViaCellClick);
        const { settings } = columnDetails;
        ColumnSettingsStyleUtils.applySettingsStyleOnCell(settings, cellElement, isHeader);
        ColumnSettingsBorderUtils.overwriteSideBorderIfSiblingsHaveSettings(columnDetails, [cellElement]); // REF-23
        const isEditable = isHeader ? !isOpenViaCellClick && settings.isHeaderTextEditable : settings.isCellTextEditable;
        CellElement.prepContentEditable(cellElement, Boolean(isEditable), isOpenViaCellClick);
        // overwritten again if static table
        if (isHeader)
            CellElement.setColumnWidth(at, cellElement, settings, at._defaultColumnsSettings);
        CellElement.setNewText(at, cellElement, text, true, false);
        return cellElement;
    }
}
CellElement.CELL_CLASS = 'cell';
CellElement.HEADER_CELL_CLASS = 'header-cell';
CellElement.HEADER_TAG = 'TH';
CellElement.DATA_TAG = 'TD';
//# sourceMappingURL=cellElement.js.map