import { LabelCellTextElement } from '../../cell/cellsWithTextDiv/selectCell/label/labelCellTextElement';
import { ElementVisibility } from '../../../utils/elements/elementVisibility';
import { ElementOffset } from '../../../utils/elements/elementOffset';
import { OverflowUtils } from '../../../utils/overflow/overflowUtils';
import { CellDropdownItemEvents } from './cellDropdownItemEvents';
import { CellDropdownScrollbar } from './cellDropdownScrollbar';
import { CellDropdownEvents } from './cellDropdownEvents';
import { CellDropdownItem } from './cellDropdownItem';
import { CellElement } from '../../cell/cellElement';
import { SIDE } from '../../../types/side';
import { Dropdown } from '../dropdown';
export class CellDropdown {
    static generateRightPosition() {
        return '4px';
    }
    // prettier-ignore
    static generateBottomPosition(cellElement, textContainerElement, borderDimensions) {
        const tableElement = cellElement.offsetParent;
        const totalVerticalBorder = borderDimensions.bottomWidth + borderDimensions.topWidth;
        const cellTop = tableElement.offsetHeight - totalVerticalBorder - cellElement.offsetTop;
        const textContainerTop = cellTop - textContainerElement.offsetTop;
        return `${textContainerTop + 6}px`;
    }
    // prettier-ignore
    static generateTopPosition(cellElement, textContainerElement, borderDimensions) {
        const topPadding = LabelCellTextElement.isLabelText(textContainerElement)
            ? textContainerElement.offsetTop + textContainerElement.offsetHeight + 2
            : cellElement.offsetHeight - 8;
        return `${ElementOffset.processTop(cellElement.offsetTop + topPadding, borderDimensions)}px`;
    }
    // prettier-ignore
    static generateLeftPosition(cellElement, textContainerElement, borderDimensions) {
        const leftPadding = LabelCellTextElement.isLabelText(textContainerElement) ? textContainerElement.offsetLeft : 1;
        return `${ElementOffset.processLeft(cellElement.offsetLeft + leftPadding, borderDimensions)}px`;
    }
    // prettier-ignore
    static correctPosition(dropdown, cellElement, textContainerElement, borderDimensions) {
        const details = ElementVisibility.getDetailsInWindow(dropdown, borderDimensions);
        if (!details.isFullyVisible) {
            if (details.blockingSides.has(SIDE.RIGHT)) {
                dropdown.style.left = '';
                // using right instead of left as it is more convenient to display dropdown beside the right side of the table
                dropdown.style.right = CellDropdown.generateRightPosition();
            }
            if (details.blockingSides.has(SIDE.BOTTOM)) {
                dropdown.style.top = '';
                // the reason why bottom property is used instead of top is because the removal of a select/label item
                // reduces the dropdown height and the bottom property keeps the dropdown position close to cell
                dropdown.style.bottom = CellDropdown.generateBottomPosition(cellElement, textContainerElement, borderDimensions);
            }
        }
    }
    // prettier-ignore
    static correctPositionForOverflow(dropdown, tableElement, overflowElement) {
        if (tableElement.offsetHeight !== overflowElement.scrollHeight) {
            dropdown.style.top = `${tableElement.offsetHeight - dropdown.offsetHeight}px`;
        }
        if (tableElement.offsetWidth !== overflowElement.scrollWidth) {
            dropdown.style.left = `${tableElement.offsetWidth - dropdown.offsetWidth}px`;
        }
    }
    static setPosition(dropdown, cellElement, borderDimensions) {
        const textContainerElement = cellElement.children[0];
        dropdown.style.bottom = '';
        dropdown.style.right = '';
        dropdown.style.left = CellDropdown.generateLeftPosition(cellElement, textContainerElement, borderDimensions);
        dropdown.style.top = CellDropdown.generateTopPosition(cellElement, textContainerElement, borderDimensions);
        const tableElement = dropdown.parentElement.parentElement;
        const overflowElement = tableElement.parentElement;
        if (OverflowUtils.isOverflowElement(overflowElement)) {
            CellDropdown.correctPositionForOverflow(dropdown, tableElement, overflowElement);
        }
        else {
            CellDropdown.correctPosition(dropdown, cellElement, textContainerElement, borderDimensions);
        }
    }
    // prettier-ignore
    static updateCellDropdown(textContainerElement, dropdown, borderDimensions, defaultText, updateCellText, matchingCellElement) {
        const textElement = CellElement.getTextElement(textContainerElement);
        CellDropdownItem.attemptHighlightMatchingItemWithCell(textElement, dropdown, defaultText, updateCellText, matchingCellElement);
        if (updateCellText) {
            CellDropdown.setPosition(dropdown.element, textElement.parentElement, borderDimensions);
        }
    }
    static focusItemOnDropdownOpen(textElement, dropdown, defaultText) {
        // the updateCellText parameter is set to false for a case where the user clicks on a select/label cell which has
        // its text with a background color but one for a select/label that has been deleted, hence we do not want to
        // highlight it with a new background color
        CellDropdownItem.attemptHighlightMatchingItemWithCell(textElement, dropdown, defaultText, false);
    }
    // prettier-ignore
    static correctWidthForOverflow(dropdownElement) {
        if (dropdownElement.clientWidth !== dropdownElement.scrollWidth) {
            const scrollbarPadding = dropdownElement.clientHeight !== dropdownElement.scrollHeight ? 16 : 0;
            const newWidth = dropdownElement.scrollWidth + scrollbarPadding;
            dropdownElement.style.width = `${Math.min(newWidth, CellDropdown.MAX_WIDTH)}px`;
        }
        // the following is a bug fix where display 'grid' property on the dropdown can set the item lengths
        // a couple of decimal places higher than clientWidth, causing an overflow when there shouldn't be
        if (dropdownElement.children.length > 0
            && dropdownElement.scrollWidth < dropdownElement.children[0].getBoundingClientRect().width) {
            dropdownElement.style.width = `${dropdownElement.clientWidth + 1}px`;
        }
    }
    static getWidth(cellElement, dropdown, dropdownStyle) {
        if (dropdownStyle === null || dropdownStyle === void 0 ? void 0 : dropdownStyle.width)
            return Number.parseInt(dropdownStyle.width);
        if (!dropdown.labelDetails)
            return Math.max(cellElement.offsetWidth - 2, CellDropdown.MIN_WIDTH);
        const textContainerElement = cellElement.children[0];
        return Math.max(cellElement.offsetWidth - textContainerElement.offsetLeft * 2, CellDropdown.MIN_WIDTH);
    }
    // prettier-ignore
    static display(at, columnIndex, cellElement) {
        const { cellDropdown, settings: { defaultText }, activeType: { cellDropdownProps } } = at._columnsDetails[columnIndex];
        const { element: dropdownEl, itemsDetails } = cellDropdown;
        if (Object.keys(itemsDetails).length > 0 && cellDropdownProps) {
            CellDropdownEvents.set(at, dropdownEl);
            CellDropdownItemEvents.blurItem(cellDropdown, 'hovered');
            CellDropdownItemEvents.blurItem(cellDropdown, 'matchingWithCellText');
            dropdownEl.style.width = `${CellDropdown.getWidth(cellElement, cellDropdown, cellDropdownProps.dropdownStyle)}px`;
            Dropdown.display(dropdownEl);
            dropdownEl.scrollLeft = 0;
            CellDropdown.correctWidthForOverflow(dropdownEl);
            CellDropdownScrollbar.setProperties(cellDropdown);
            CellDropdown.setPosition(dropdownEl, cellElement, at._tableDimensions.border);
            const textElement = cellElement.children[0];
            CellDropdown.focusItemOnDropdownOpen(textElement, cellDropdown, defaultText);
            return true;
        }
        return false;
    }
    static setCustomStyle(cellDropdown, dropdownStyle) {
        const { paddingTop, paddingBottom, marginTop, marginLeft, border, textAlign } = dropdownStyle;
        cellDropdown.element.style.paddingTop = paddingTop || Dropdown.DROPDOWN_VERTICAL_PX;
        cellDropdown.element.style.paddingBottom = paddingBottom || Dropdown.DROPDOWN_VERTICAL_PX;
        cellDropdown.element.style.marginTop = marginTop || '0px';
        cellDropdown.element.style.marginLeft = marginLeft || '0px';
        cellDropdown.element.style.border = border || 'none';
        cellDropdown.element.style.textAlign = textAlign || 'left';
    }
    static setCustomState(cellDropdown, cellDropdownType) {
        cellDropdown.customDropdownStyle = cellDropdownType.dropdownStyle;
        cellDropdown.customItemStyle = cellDropdownType.optionStyle;
        cellDropdown.canAddMoreOptions = !!cellDropdownType.canAddMoreOptions;
    }
    // prettier-ignore
    static setUpDropdown(at, columnIndex) {
        const { _columnsDetails, _globalItemColors } = at;
        const { activeType: { cellDropdownProps }, cellDropdown } = _columnsDetails[columnIndex];
        if (!cellDropdownProps)
            return;
        // REF-34
        cellDropdown.labelDetails = cellDropdownProps.isBasicSelect ? undefined : { globalItemColors: _globalItemColors };
        CellDropdown.setCustomState(cellDropdown, cellDropdownProps);
        CellDropdownItem.populateItems(at, columnIndex);
        if (cellDropdownProps.dropdownStyle)
            CellDropdown.setCustomStyle(cellDropdown, cellDropdownProps.dropdownStyle);
    }
    // REF-8 - Created for every column
    static createAndAppend(containerElement) {
        const dropdownElement = Dropdown.createBase();
        dropdownElement.style.maxHeight = CellDropdown.MAX_HEIGHT_PX;
        dropdownElement.classList.add(CellDropdown.CELL_DROPDOWN_CLASS);
        containerElement.appendChild(dropdownElement);
        return dropdownElement;
    }
    static getDefaultObj(dropdownElement) {
        return {
            itemsDetails: {},
            activeItems: {},
            element: dropdownElement,
            canAddMoreOptions: true,
            scrollbarPresence: {
                horizontal: false,
                vertical: false,
            },
        };
    }
    static createContainerElement() {
        return document.createElement('div');
    }
}
CellDropdown.CELL_DROPDOWN_CLASS = 'cell-dropdown';
CellDropdown.MAX_HEIGHT_PX = '147px';
CellDropdown.MIN_WIDTH = 70;
CellDropdown.MAX_WIDTH = 200;
//# sourceMappingURL=cellDropdown.js.map