import { GenericElementUtils } from '../../utils/elements/genericElementUtils';
import { SVGIconUtils } from '../../utils/svgIcons/svgIconUtils';
import { DropdownItemEvents } from './dropdownItemEvents';
export class DropdownItem {
    static toggleItem(item, isDisplay) {
        item.style.display = isDisplay ? DropdownItem.DISPLAY : DropdownItem.HIDDEN;
    }
    static isDisplayed(item) {
        return item.style.display === DropdownItem.DISPLAY;
    }
    static createDropdownItemBaseElement(tag) {
        const dropdownItemBaseDiv = document.createElement(tag);
        dropdownItemBaseDiv.classList.add(DropdownItem.DROPDOWN_ITEM_IDENTIFIER);
        return dropdownItemBaseDiv;
    }
    static createItem(dropdownElement) {
        const itemElement = DropdownItem.createDropdownItemBaseElement('div');
        if (dropdownElement)
            itemElement.tabIndex = dropdownElement.children.length;
        itemElement.classList.add(DropdownItem.DROPDOWN_ITEM_CLASS, GenericElementUtils.NOT_SELECTABLE_CLASS);
        return itemElement;
    }
    // no need to sanitize paste as input element already does it
    static addInputItem(at, dropdownElement) {
        const itemElement = DropdownItem.createItem(dropdownElement);
        itemElement.classList.add(DropdownItem.DROPDOWN_INPUT_ITEM_CLASS);
        const inputElement = DropdownItem.createDropdownItemBaseElement('input');
        inputElement.classList.add(DropdownItem.DROPDOWN_INPUT_CLASS);
        itemElement.appendChild(inputElement);
        dropdownElement.appendChild(itemElement);
        DropdownItemEvents.addItemEvents(at._activeOverlayElements, inputElement);
    }
    // REF-10
    static insertIcon(buttonElement, iconSettings) {
        const { svgString, containerStyles } = iconSettings;
        const container = document.createElement('div');
        container.classList.add(DropdownItem.DROPDOWN_ITEM_ICON_CONTAINER_CLASS);
        Object.assign(container.style, containerStyles === null || containerStyles === void 0 ? void 0 : containerStyles.dropdown);
        const svgIconElement = SVGIconUtils.createSVGElement(svgString);
        // using style as the class has no effect on svg
        svgIconElement.style.filter = SVGIconUtils.DROPDOWN_ITEM_FILTER;
        container.appendChild(svgIconElement);
        buttonElement.insertBefore(container, buttonElement.children[0]);
    }
    static addPlaneButtonItem(dropdownElement, text, index) {
        const itemElement = DropdownItem.createItem(dropdownElement);
        const textElement = DropdownItem.createDropdownItemBaseElement('div');
        textElement.innerText = text || DropdownItem.DROPDOWN_ITEM_EMPTY_PLACEHOLDER_TEXT;
        if (text.trim() === '')
            textElement.classList.add(DropdownItem.DROPDOWN_ITEM_EMPTY_CLASS);
        itemElement.append(textElement);
        if (dropdownElement) {
            if (index !== undefined && dropdownElement.children[index]) {
                dropdownElement.insertBefore(itemElement, dropdownElement.children[index]);
            }
            else {
                dropdownElement.appendChild(itemElement);
            }
        }
        return itemElement;
    }
    // prettier-ignore
    static createButtonItemNoEvents(dropdown, itemSettings, ...classNames) {
        const buttonElement = DropdownItem.addPlaneButtonItem(dropdown, itemSettings.text);
        if (itemSettings.iconSettings)
            DropdownItem.insertIcon(buttonElement, itemSettings.iconSettings);
        if (classNames.length > 0)
            buttonElement.classList.add(...classNames);
        return buttonElement;
    }
    static addTitle(dropdownElement, text) {
        const titleElement = DropdownItem.createDropdownItemBaseElement('div');
        titleElement.classList.add(DropdownItem.DROPDOWN_ITEM_CLASS, DropdownItem.DROPDOWN_TITLE_ITEM_CLASS);
        titleElement.innerText = text;
        dropdownElement.appendChild(titleElement);
    }
    static addDivider(dropdownElement) {
        const dividerElement = DropdownItem.createDropdownItemBaseElement('div');
        dividerElement.classList.add(DropdownItem.DROPDOWN_ITEM_DIVIDER_CLASS);
        dropdownElement.appendChild(dividerElement);
    }
    // prettier-ignore
    static addButtonItem(at, dropdown, itemSettings, ...classNames) {
        const buttonElement = DropdownItem.createButtonItemNoEvents(dropdown, itemSettings, ...classNames);
        DropdownItemEvents.addItemEvents(at._activeOverlayElements, buttonElement);
        return buttonElement;
    }
    // prettier-ignore
    static addNewButtonItems(at, dropdownElement, itemsSettings) {
        return itemsSettings.map((item) => {
            return DropdownItem.addButtonItem(at, dropdownElement, item);
        });
    }
    static addButtonItemElements(at, dropdownElement, elements) {
        elements.forEach((element) => {
            element.tabIndex = dropdownElement.children.length;
            dropdownElement.appendChild(element);
            DropdownItemEvents.addItemEvents(at._activeOverlayElements, element);
        });
    }
    static removeItems(dropdown) {
        Array.from(dropdown.children).forEach((item) => item.remove());
    }
    static doesElementContainItemClass(element) {
        return element.classList.contains(DropdownItem.DROPDOWN_ITEM_IDENTIFIER);
    }
    static doesElementContainInputClass(element) {
        return element.classList.contains(DropdownItem.DROPDOWN_INPUT_CLASS);
    }
    static getInputElement(dropdownElement) {
        return dropdownElement.getElementsByClassName(DropdownItem.DROPDOWN_INPUT_ITEM_CLASS)[0];
    }
    static toggleUsability(item, isUsable) {
        const icon = item.children[0];
        if (isUsable) {
            item.classList.remove(DropdownItem.DISABLED_ITEM_CLASS);
            icon.style.filter = '';
        }
        else {
            item.classList.add(DropdownItem.DISABLED_ITEM_CLASS);
            icon.style.filter = SVGIconUtils.LIGHT_GREY_FILTER;
        }
    }
}
DropdownItem.DROPDOWN_ITEM_CLASS = 'dropdown-item';
DropdownItem.DISABLED_ITEM_CLASS = 'dropdown-disabled-item';
DropdownItem.ACTIVE_ITEM_CLASS = 'active-dropdown-item';
DropdownItem.DROPDOWN_INPUT_CLASS = 'dropdown-input';
DropdownItem.DROPDOWN_ITEM_ICON_CONTAINER_CLASS = 'dropdown-item-icon-container';
DropdownItem.DROPDOWN_INPUT_ITEM_CLASS = 'dropdown-input-item';
DropdownItem.DROPDOWN_TITLE_ITEM_CLASS = 'dropdown-title-item';
DropdownItem.DROPDOWN_ITEM_DIVIDER_CLASS = 'dropdown-item-divider';
DropdownItem.DROPDOWN_ITEM_EMPTY_CLASS = 'dropdown-item-empty';
DropdownItem.DROPDOWN_ITEM_EMPTY_PLACEHOLDER_TEXT = '...';
// this is used to identify if a mouse event is currently on a dropdown item
DropdownItem.DROPDOWN_ITEM_IDENTIFIER = 'dropdown-item-identifier';
DropdownItem.HIDDEN = 'none';
DropdownItem.DISPLAY = '';
//# sourceMappingURL=dropdownItem.js.map