import { DropdownItemHighlightUtils } from '../../../utils/color/dropdownItemHighlightUtils';
import { FrameComponentsColors } from '../../../utils/frameComponents/frameComponentsColors';
import { FullTableOverlayElement } from '../../fullTableOverlay/fullTableOverlayElement';
import { FocusedCellUtils } from '../../../utils/focusedElements/focusedCellUtils';
import { ElementVisibility } from '../../../utils/elements/elementVisibility';
import { CellHighlightUtils } from '../../../utils/color/cellHighlightUtils';
import { ElementOffset } from '../../../utils/elements/elementOffset';
import { Browser } from '../../../utils/browser/browser';
import { RowDropdownEvents } from './rowDropdownEvents';
import { CellElement } from '../../cell/cellElement';
import { RowDropdownItem } from './rowDropdownItem';
import { SIDE } from '../../../types/side';
import { Dropdown } from '../dropdown';
export class RowDropdown {
    // prettier-ignore
    static hide(at) {
        const { _activeOverlayElements: { rowDropdown, fullTableOverlay }, _focusedElements: { cell: { element, rowIndex } }, _frameComponents: { cellColors, displayIndexColumn } } = at;
        if (!rowDropdown || !fullTableOverlay || !element)
            return;
        Dropdown.hide(rowDropdown, fullTableOverlay);
        const colors = FrameComponentsColors.getColorsBasedOnParam(cellColors, rowIndex);
        if (displayIndexColumn)
            CellHighlightUtils.fade(element, colors.default);
        DropdownItemHighlightUtils.fadeCurrentlyHighlighted(at._activeOverlayElements);
        setTimeout(() => {
            // in a timeout because upon pressing esc/enter key on dropdown, the window event is fired after which checks it
            delete at._focusedElements.rowDropdown;
            FocusedCellUtils.purge(at._focusedElements.cell);
        });
    }
    static focusCell(at, rowIndex, cellElement) {
        const { _frameComponents, _focusedElements } = at;
        if (_frameComponents.displayIndexColumn) {
            FocusedCellUtils.setIndexCell(_focusedElements.cell, cellElement, rowIndex);
        }
        else {
            FocusedCellUtils.set(_focusedElements.cell, cellElement, rowIndex, 0);
        }
    }
    // prettier-ignore
    static correctPositionWhenBottomOverflow(tableDimensions, dropdown, initialTopValue) {
        const tableTopOffset = dropdown.parentElement.offsetTop + tableDimensions.border.topWidth;
        let newTopValue = window.pageYOffset + window.innerHeight - tableTopOffset - dropdown.offsetHeight;
        if (Browser.IS_FIREFOX)
            newTopValue += tableDimensions.border.topWidth;
        dropdown.style.top = `${newTopValue}px`;
        const visibilityDetails = ElementVisibility.getDetailsInWindow(dropdown, tableDimensions.border);
        if (!visibilityDetails.isFullyVisible && visibilityDetails.blockingSides.has(SIDE.TOP)) {
            dropdown.style.top = initialTopValue;
        }
    }
    static getLeft(at, cellElement) {
        var _a;
        const cellClick = (_a = at.rowDropdown.displaySettings.openMethod) === null || _a === void 0 ? void 0 : _a.cellClick;
        return `${ElementOffset.processWidth(cellClick ? cellElement.offsetWidth : 5, at._tableDimensions.border)}px`;
    }
    static displayAndSetPosition(at, cellElement, dropdown) {
        const initialTopValue = `${ElementOffset.processTop(cellElement.offsetTop, at._tableDimensions.border)}px`;
        dropdown.style.top = initialTopValue;
        dropdown.style.left = RowDropdown.getLeft(at, cellElement);
        // needs to be displayed here to evalute if in view port
        Dropdown.display(dropdown);
        const visibilityDetails = ElementVisibility.getDetailsInWindow(dropdown, at._tableDimensions.border);
        if (!visibilityDetails.isFullyVisible) {
            if (visibilityDetails.blockingSides.has(SIDE.BOTTOM)) {
                RowDropdown.correctPositionWhenBottomOverflow(at._tableDimensions, dropdown, initialTopValue);
            }
        }
    }
    // prettier-ignore
    static setOverflowPosition(at, cellElement, dropdown, overflowElement) {
        if (at._stickyProps.header && cellElement.tagName === CellElement.HEADER_TAG) {
            if (at._overflow) {
                dropdown.style.top = `${overflowElement.scrollTop}px`;
            }
            else {
                const rowOffset = cellElement.parentElement.offsetTop;
                const borderTopWidth = Number.parseInt(getComputedStyle(cellElement).borderTopWidth);
                dropdown.style.top = `${rowOffset + borderTopWidth}px`; // REF-37
            }
        }
        else {
            dropdown.style.top = `${ElementOffset.processTop(cellElement.offsetTop, at._tableDimensions.border)}px`;
        }
        dropdown.style.left = RowDropdown.getLeft(at, cellElement);
    }
    // no active table based overflow
    static displayAndSetPositionForSticky(at, cellElement, dropdown) {
        const overflowElement = at.parentElement;
        RowDropdown.setOverflowPosition(at, cellElement, dropdown, overflowElement);
        Dropdown.display(dropdown);
    }
    static displayAndSetPositionOverflow(at, cellElement, dropdown) {
        const { _tableElementRef, _overflow, _stickyProps } = at;
        if (!_tableElementRef || !(_overflow === null || _overflow === void 0 ? void 0 : _overflow.overflowContainer))
            return;
        RowDropdown.setOverflowPosition(at, cellElement, dropdown, _overflow.overflowContainer);
        // needs to be displayed here to evalute if overflow
        Dropdown.display(dropdown);
        const isStickyCell = _stickyProps.header && cellElement.tagName === CellElement.HEADER_TAG;
        if (_tableElementRef.offsetHeight !== _overflow.overflowContainer.scrollHeight && !isStickyCell) {
            dropdown.style.top = `${_tableElementRef.offsetHeight - dropdown.offsetHeight}px`;
        }
    }
    static display(rowIndex, cellElement) {
        var _a;
        const dropdownElement = this._activeOverlayElements.rowDropdown;
        RowDropdownItem.update(this, dropdownElement, rowIndex);
        if ((_a = this._overflow) === null || _a === void 0 ? void 0 : _a.overflowContainer) {
            RowDropdown.displayAndSetPositionOverflow(this, cellElement, dropdownElement);
        }
        else if (this._stickyProps.header) {
            RowDropdown.displayAndSetPositionForSticky(this, cellElement, dropdownElement);
        }
        else {
            RowDropdown.displayAndSetPosition(this, cellElement, dropdownElement);
        }
        FullTableOverlayElement.display(this);
        setTimeout(() => RowDropdown.focusCell(this, rowIndex, cellElement));
    }
    static create(at) {
        const dropdownElement = Dropdown.createBase();
        RowDropdownEvents.set(at, dropdownElement);
        RowDropdownItem.setUpItems(at, dropdownElement);
        return dropdownElement;
    }
}
//# sourceMappingURL=rowDropdown.js.map