import { ColumnSettingsFrameBorderUtils } from './columnSettingsFrameBorderUtils';
import { ColumnSettingsStyleUtils } from './columnSettingsStyleUtils';
import { ElementStyle } from '../elements/elementStyle';
// REF-23
export class ColumnSettingsBorderUtils {
    // REF-23
    static overwriteSideBorderIfSiblingsHaveSettings(columnDetails, cellElements) {
        const { left, right } = columnDetails.bordersOverwrittenBySiblings;
        cellElements.forEach((cellElement) => {
            if (left)
                cellElement.style.borderLeftWidth = ColumnSettingsBorderUtils.UNSET_PX;
            if (right)
                cellElement.style.borderRightWidth = ColumnSettingsBorderUtils.UNSET_PX;
        });
    }
    // prettier-ignore
    static getColumnBorderStyles(subjectBorder) {
        const subjectBorderStyle = subjectBorder === 'left' ? 'borderLeftWidth' : 'borderRightWidth';
        const siblingBorderStyle = subjectBorder === 'left' ? 'borderRightWidth' : 'borderLeftWidth';
        return { subjectBorderStyle, siblingBorderStyle };
    }
    static isBorderDisplayed(cell, borderStyle) {
        return Boolean(cell.style[borderStyle] && cell.style[borderStyle] !== ColumnSettingsBorderUtils.UNSET_PX);
    }
    // REF-23
    // prettier-ignore
    static unsetSubjectBorder(subjectElements, siblingElements, subjectBorder, comparisonRow, bordersOverwrittenBySiblings) {
        const { subjectBorderStyle, siblingBorderStyle } = ColumnSettingsBorderUtils.getColumnBorderStyles(subjectBorder);
        const subjectHeader = subjectElements[comparisonRow];
        const siblingHeader = siblingElements[comparisonRow];
        // only unset if right and left are present on the sibling cells and vice/versa
        if (ColumnSettingsBorderUtils.isBorderDisplayed(subjectHeader, subjectBorderStyle)
            && ColumnSettingsBorderUtils.isBorderDisplayed(siblingHeader, siblingBorderStyle)) {
            if (bordersOverwrittenBySiblings)
                bordersOverwrittenBySiblings[subjectBorder] = true;
            subjectElements.forEach((element) => {
                ElementStyle.setStyle(element, subjectBorderStyle, ColumnSettingsBorderUtils.UNSET_PX);
            });
        }
    }
    // prettier-ignore
    static unsetColumnBorder(column, siblingColumn, subjectBorder) {
        ColumnSettingsBorderUtils.unsetSubjectBorder(column.elements, siblingColumn.elements, subjectBorder, 0, column.bordersOverwrittenBySiblings);
    }
    // if current column and sibling have custom setting styles
    // REF-23
    // prettier-ignore
    static unsetBorders(currentColumnDetails, leftColumnDetails, rightColumnDetails) {
        if (!currentColumnDetails)
            return;
        // if current column has custom style precedence
        if (currentColumnDetails.settings.stylePrecedence) {
            // remove the left column right border
            if (leftColumnDetails) {
                ColumnSettingsBorderUtils.unsetColumnBorder(leftColumnDetails, currentColumnDetails, 'right');
            }
            if (rightColumnDetails) {
                // if right has precedence, remove current column right border
                if (rightColumnDetails.settings.stylePrecedence) {
                    ColumnSettingsBorderUtils.unsetColumnBorder(currentColumnDetails, rightColumnDetails, 'right');
                    // if right does not have precedence, remove right column left border
                }
                else if (!rightColumnDetails.settings.stylePrecedence) {
                    ColumnSettingsBorderUtils.unsetColumnBorder(rightColumnDetails, currentColumnDetails, 'left');
                }
            }
            // if current column does not have a custom style but siblings do
        }
        else {
            if (rightColumnDetails && rightColumnDetails.settings.stylePrecedence) {
                ColumnSettingsBorderUtils.unsetColumnBorder(currentColumnDetails, rightColumnDetails, 'right');
            }
            if (leftColumnDetails && leftColumnDetails.settings.stylePrecedence) {
                ColumnSettingsBorderUtils.unsetColumnBorder(currentColumnDetails, leftColumnDetails, 'left');
            }
        }
    }
    static resetBorderOverwritingState(currentColumn) {
        if (!currentColumn)
            return;
        currentColumn.bordersOverwrittenBySiblings.left = false;
        currentColumn.bordersOverwrittenBySiblings.right = false;
    }
    // prettier-ignore
    static resetIfBorderOverwritten(at, columnIndex, subjectColumn, subjectBorder) {
        if (subjectColumn === null || subjectColumn === void 0 ? void 0 : subjectColumn.bordersOverwrittenBySiblings[subjectBorder]) {
            subjectColumn.bordersOverwrittenBySiblings[subjectBorder] = false;
            // not necessarily changing the style, but need to run this as there are a lot of styles unset and a lot being set
            ColumnSettingsStyleUtils.changeStyleFunc(at, columnIndex, at._columnsDetails[columnIndex].settings);
        }
    }
    // REF-23
    static updateSiblingColumns(at, columnIndex) {
        const { _columnsDetails } = at;
        const currentColumnDetails = _columnsDetails[columnIndex];
        const leftColumnDetails = _columnsDetails[columnIndex - 1];
        const rightColumnDetails = _columnsDetails[columnIndex + 1];
        // reset sibling columns if their borders were previously overwritten
        ColumnSettingsBorderUtils.resetIfBorderOverwritten(at, columnIndex + 1, rightColumnDetails, 'left');
        ColumnSettingsBorderUtils.resetIfBorderOverwritten(at, columnIndex - 1, leftColumnDetails, 'right');
        // because current column is reanalyzed, need to reset state as some props may no longer be true
        ColumnSettingsBorderUtils.resetBorderOverwritingState(currentColumnDetails);
        // overwrite borders preceded by settings style
        ColumnSettingsBorderUtils.unsetBorders(currentColumnDetails, leftColumnDetails, rightColumnDetails);
        // overwrite frame cell borders preceded by settings style
        ColumnSettingsFrameBorderUtils.updateFrameColumns(at, currentColumnDetails, leftColumnDetails, rightColumnDetails);
    }
}
ColumnSettingsBorderUtils.UNSET_PX = '0px';
//# sourceMappingURL=columnSettingsBorderUtils.js.map