import { ColumnSettingsBorderUtils } from '../columnSettings/columnSettingsBorderUtils';
import { ResetColumnStyles } from '../columnSettings/resetColumnStyles';
import { CellElement } from '../../elements/cell/cellElement';
// this class only operates on data cells
// REF-3
export class ProcessedDataTextStyle {
    // prettier-ignore
    static setCustomStyle(changeStyleFunc, text, columnDetails, rowIndex, processedStyle, textContainerElement, columnsSettings) {
        if (changeStyleFunc) {
            ResetColumnStyles.setDefaultStyle(columnDetails, processedStyle, textContainerElement, columnsSettings);
            const newStyle = changeStyleFunc(String(text), rowIndex);
            Object.assign(textContainerElement.style, newStyle);
            processedStyle.lastAppliedStyle = newStyle;
            ColumnSettingsBorderUtils.overwriteSideBorderIfSiblingsHaveSettings(columnDetails, [textContainerElement]);
        }
    }
    // prettier-ignore
    static setFailedValidationStyle(columnDetails, processedStyle, textContainerElement) {
        const { textValidation } = columnDetails.activeType;
        const failedStyle = textValidation.failedStyle || ProcessedDataTextStyle.DEFAULT_FAILED_VALIDATION_STYLE;
        Object.assign(textContainerElement.style, failedStyle);
        processedStyle.lastAppliedStyle = failedStyle;
    }
    // prettier-ignore
    static setStyle(isValid, columnDetails, processedStyle, customTextProcessing, textContainerElement, columnsSettings) {
        let wasValidationStyleSet = false; // REF-3
        if (!isValid) {
            if (customTextProcessing === null || customTextProcessing === void 0 ? void 0 : customTextProcessing.changeStyleFunc) {
                ResetColumnStyles.setDefaultStyle(columnDetails, processedStyle, textContainerElement, columnsSettings);
            }
            ProcessedDataTextStyle.setFailedValidationStyle(columnDetails, processedStyle, textContainerElement);
            wasValidationStyleSet = true;
        }
        else if (!(customTextProcessing === null || customTextProcessing === void 0 ? void 0 : customTextProcessing.changeStyleFunc)) { // REF-3 - set to default if no custom processing will occur
            ResetColumnStyles.setDefaultStyle(columnDetails, processedStyle, textContainerElement, columnsSettings);
            processedStyle.lastAppliedStyle = {};
            wasValidationStyleSet = true;
        }
        if (wasValidationStyleSet) {
            ColumnSettingsBorderUtils.overwriteSideBorderIfSiblingsHaveSettings(columnDetails, [textContainerElement]);
        }
        return wasValidationStyleSet;
    }
    // prettier-ignore
    static setCellStyle(at, rowIndex, columnIndex, overwrite = false) {
        const columnDetails = at._columnsDetails[columnIndex];
        const textContainerElement = columnDetails.elements[rowIndex];
        const processedStyle = columnDetails.processedStyle[rowIndex];
        const text = CellElement.getText(textContainerElement);
        const { textValidation: { func: validationFunc }, customTextProcessing } = columnDetails.activeType;
        let wasValidationStyleSet = false; // REF-3
        if (validationFunc) {
            const isValid = validationFunc(text);
            if (overwrite || processedStyle.isValid !== isValid) {
                wasValidationStyleSet = ProcessedDataTextStyle.setStyle(isValid, columnDetails, processedStyle, customTextProcessing, textContainerElement, at._defaultColumnsSettings);
                processedStyle.isValid = isValid;
            }
        }
        if (!wasValidationStyleSet && (customTextProcessing === null || customTextProcessing === void 0 ? void 0 : customTextProcessing.changeStyleFunc)) { // REF-3
            ProcessedDataTextStyle.setCustomStyle(customTextProcessing.changeStyleFunc, text, columnDetails, rowIndex, processedStyle, textContainerElement, at._defaultColumnsSettings);
        }
    }
    static setStyleOnColumn(at, columnIndex) {
        const columnDetails = at._columnsDetails[columnIndex];
        columnDetails.elements.slice(1).forEach((element, rowIndex) => {
            const relativeRowIndex = rowIndex + 1;
            ProcessedDataTextStyle.setCellStyle(at, relativeRowIndex, columnIndex, true);
            ColumnSettingsBorderUtils.overwriteSideBorderIfSiblingsHaveSettings(columnDetails, [element]);
        });
    }
    static unsetStyleOnColumn(at, columnIndex, oldCellStyle) {
        const columnDetails = at._columnsDetails[columnIndex];
        columnDetails.elements.slice(1).forEach((element, rowIndex) => {
            const relativeRowIndex = rowIndex + 1;
            const processedStyle = columnDetails.processedStyle[relativeRowIndex];
            ResetColumnStyles.setDefaultStyle(columnDetails, processedStyle, element, at._defaultColumnsSettings, oldCellStyle);
        });
    }
    // using this to first unset the previous processed style, allow new settings/type to be applied and then set
    // new style
    // prettier-ignore
    static resetDataCellsStyle(at, columnIndex, changeFunc, oldCellStyle) {
        ProcessedDataTextStyle.unsetStyleOnColumn(at, columnIndex, oldCellStyle);
        changeFunc(); // arguments are expected to be binded
        ProcessedDataTextStyle.setStyleOnColumn(at, columnIndex);
    }
    // prettier-ignore
    // this is used for a case where the default style has been set and need to reapply the processed style
    // without having to rerun the validation/changeStyleFunc functions
    static reapplyCellsStyle(at, columnIndex) {
        const columnDetails = at._columnsDetails[columnIndex];
        const { textValidation: { func: validationFunc }, customTextProcessing } = columnDetails.activeType;
        if (validationFunc || (customTextProcessing === null || customTextProcessing === void 0 ? void 0 : customTextProcessing.changeStyleFunc)) {
            columnDetails.elements.slice(1).forEach((element, rowIndex) => {
                const relativeRowIndex = rowIndex + 1;
                Object.assign(element.style, columnDetails.processedStyle[relativeRowIndex].lastAppliedStyle);
            });
        }
    }
    static getDefaultProcessedTextStyle() {
        return { isValid: true, lastAppliedStyle: {} };
    }
}
ProcessedDataTextStyle.DEFAULT_FAILED_VALIDATION_STYLE = { color: 'grey' };
//# sourceMappingURL=processedDataTextStyle.js.map