import { DEFAULT_COLUMN_TYPES } from '../../enums/defaultColumnTypes';
import { CellElementIndex } from '../elements/cellElementIndex';
import { CellEvents } from '../../elements/cell/cellEvents';
import { ColumnTypesUtils } from './columnTypesUtils';
import { FireEvents } from '../events/fireEvents';
import { RegexUtils } from '../regex/regexUtils';
export class Sort {
    static extractNumberFromString(text) {
        const numberStringArr = RegexUtils.extractFloatStrs(text);
        if (numberStringArr && numberStringArr.length > 0) {
            return Number(numberStringArr[0]);
        }
        return 0;
    }
    // cannot safely identify if nothing has been changed, hence need to send out an update for all cells
    // prettier-ignore
    static update(at, sortedDataContent) {
        const { _tableBodyElementRef, _frameComponents: { displayIndexColumn }, content } = at;
        const rowElements = _tableBodyElementRef.children;
        sortedDataContent.forEach((row, rowIndex) => {
            const relativeRowIndex = rowIndex + 1;
            const rowChildren = rowElements[relativeRowIndex].children;
            row.forEach((cell, columnIndex) => {
                const elementColumnIndex = CellElementIndex.getViaColumnIndex(columnIndex, !!displayIndexColumn);
                const cellElement = rowChildren[elementColumnIndex];
                // the reason why updateContent property is set to false is because we do not want to overwrite content array
                // cells as their row references are still the same with the sortedDataContent, hence upon attempting to
                // overwrite the content array cells, sortedDataContent cells are also overwritten. This is problematic because
                // sortedDataContent rows are in a different order, hence the cells to be traversed can already be overwritten
                // by the earlier cells
                CellEvents.updateCell(at, cell, relativeRowIndex, columnIndex, { processText: false, element: cellElement, updateTableEvent: false, updateContent: false });
                ColumnTypesUtils.updateDataElements(at, rowIndex, columnIndex, cellElement);
            });
        });
        content.splice(1, sortedDataContent.length, ...sortedDataContent);
        setTimeout(() => FireEvents.onContentUpdate(at));
    }
    static sortStringsColumnAscending(content, columnIndex) {
        content.sort((a, b) => String(a[columnIndex]).localeCompare(String(b[columnIndex])));
    }
    static sortStringsColumnDescending(content, columnIndex) {
        content.sort((a, b) => String(b[columnIndex]).localeCompare(String(a[columnIndex])));
    }
    static sortStrings(dataContent, columnIndex, isAsc) {
        if (isAsc) {
            Sort.sortStringsColumnAscending(dataContent, columnIndex);
        }
        else {
            Sort.sortStringsColumnDescending(dataContent, columnIndex);
        }
    }
    // prettier-ignore
    static parseComparedText(cellText1, cellText2, isAsc, parse) {
        // the number result follows JavaScript's sort standard that will cause incorrect text to move to the bottom
        const parsedCellText1 = parse(cellText1);
        if (parsedCellText1 === undefined)
            return isAsc ? 1 : -1;
        const parsedCellText2 = parse(cellText2);
        if (parsedCellText2 === undefined)
            return isAsc ? -1 : 1;
        return [parsedCellText1, parsedCellText2];
    }
    static validateType(validate, cellText) {
        return validate === undefined || validate(cellText) ? cellText : undefined;
    }
    // prettier-ignore
    static validateAndSort(cellText1, cellText2, sorting, validate, isAsc) {
        const parseResult = Sort.parseComparedText(cellText1, cellText2, isAsc, Sort.validateType.bind(this, validate));
        if (typeof parseResult === 'number')
            return parseResult;
        const sortFunc = isAsc ? sorting.ascendingFunc : sorting.descendingFunc;
        return sortFunc(parseResult[0], parseResult[1]);
    }
    static sortViaSortFuncs(type, dataContent, colIndex, isAsc) {
        const { sorting, textValidation } = type;
        if (!sorting)
            return;
        dataContent.sort((a, b) => Sort.validateAndSort(a[colIndex], b[colIndex], sorting, textValidation.func, isAsc));
    }
    static compareDates(ymd1, ymd2) {
        return new Date(...ymd1) - new Date(...ymd2);
    }
    static parseYMDFormat(validate, calendar, cellText) {
        const isValid = Sort.validateType(validate, cellText);
        return isValid ? calendar.toYMDFunc(cellText) : undefined;
    }
    // prettier-ignore
    static sortDates(type, dataContent, columnIndex, isAsc) {
        const { calendar, textValidation } = type;
        if (!calendar)
            return;
        dataContent.sort((a, b) => {
            // isAsc param is always true because the order at which we pass in text is always the same as the asc sort
            const parseResult = Sort.parseComparedText(a[columnIndex], b[columnIndex], true, Sort.parseYMDFormat.bind(this, textValidation.func, calendar));
            if (typeof parseResult === 'number')
                return parseResult;
            return isAsc
                ? Sort.compareDates(parseResult[0], parseResult[1])
                : Sort.compareDates(parseResult[1], parseResult[0]);
        });
    }
    static sortColumn(at, columnIndex, isAsc) {
        const dataContent = at.content.slice(1);
        const { activeType } = at._columnsDetails[columnIndex];
        if (activeType.calendar) {
            Sort.sortDates(activeType, dataContent, columnIndex, isAsc);
        }
        else if (activeType.sorting) {
            Sort.sortViaSortFuncs(activeType, dataContent, columnIndex, isAsc);
        }
        else {
            Sort.sortStrings(dataContent, columnIndex, isAsc);
        }
        Sort.update(at, dataContent);
    }
}
Sort.DEFAULT_TYPES_SORT_FUNCS = {
    [DEFAULT_COLUMN_TYPES.NUMBER]: {
        ascendingFunc: (cellText1, cellText2) => {
            return Number(cellText1) - Number(cellText2);
        },
        descendingFunc: (cellText1, cellText2) => Number(cellText2) - Number(cellText1),
    },
    [DEFAULT_COLUMN_TYPES.CURRENCY]: {
        ascendingFunc: (cellText1, cellText2) => {
            return Sort.extractNumberFromString(cellText1) - Sort.extractNumberFromString(cellText2);
        },
        descendingFunc: (cellText1, cellText2) => {
            return Sort.extractNumberFromString(cellText2) - Sort.extractNumberFromString(cellText1);
        },
    },
};
//# sourceMappingURL=sort.js.map